% Correction application Chapitre 3
% Macroconomie quantitative (2014, d. Ellipses) 
% par Christophe Cahn

% Ce fichier permet de reproduire les lements contenu
% dans le fichier App_3_cycle.pdf disponible sur le site
% www.christophecahn.fr/macroquant

% Filename : App_3_cycle.m
% Application : Matlab/Octave
% Date : 24-04-2014
% Christophe Cahn (c) 2014

addpath('../MMMtbx');
		
clear all; clc; close all;

% 1.a
load('../data/USdata.txt');
t = USdata(:,1);
YUS = USdata(:,2);

% 1.b
yUS = log(YUS);
plot(t,yUS);
print('./App_3_yUS.pdf','-dpdf')

% 1.c
gy = YUS(2:end)./YUS(1:end-1)-1;
plot(t(2:end),gy);
print('./App_3_gy.pdf','-dpdf')

% 1.d
lgy = diff(yUS);
errmax = max(abs(lgy-gy));
fprintf('erreur maximum : %f\n',errmax);

% 1.e
YUSa = YUS(4:end) + YUS(3:end-1) + YUS(2:end-2) + YUS(1:end-3) ;
gya = YUSa(5:end)./YUSa(1:end-4) - 1 ;

gya2 = .25*gy(1:end-6) + .5*gy(2:end-5) + .75*gy(3:end-4) + gy(4:end-3)	+ .75*gy(5:end-2) + .5*gy(6:end-1) + .25*gy(7:end) ;

t2 = (1:length(gya)) ;
plot(t2,gya,'r',t2,gya2,'k');
legend('calcul exact','approximation');
title('Taux de croissance annuel');
print('./App_3_gya.pdf','-dpdf')

% 1.f
res = tpy(yUS,t,'US')
print('./App_3_tpy.pdf','-dpdf')

res.APT/res.DPT*4*100

% 2.a
T = length(yUS);
trend = (1:T)' ;

% 2.b
cst = ones(T,1);
X = [trend cst];
Y = yUS;
b = (X'*X)\(X'*Y)

% 2.c
tb1 = 1966.5 ; % 1966-III
tb2 = 1996.0 ; % 1996-I
 
y1 = yUS(t<=tb1) ;
y2 = yUS((t>tb1)&(t<=tb2)) ;
y3 = yUS(t>tb2) ;
n1 = length(y1) ;
n2 = length(y2) ;
n3 = length(y3) ;
x1 = [(1:n1)' ones(n1,1)] ;
x2 = [(1:n2)' ones(n2,1)] ;
x3 = [(1:n3)' ones(n3,1)] ;

ols1 = ols(y1,x1);
prt(ols1); figure(2); plt(ols1); xlabel('1947:I-1966:III');
print('./App_3_ols1.pdf','-dpdf')
ols2 = ols(y2,x2);
prt(ols2); figure(3); plt(ols2); xlabel('1966:IV-1996:I');
print('./App_3_ols2.pdf','-dpdf')
ols3 = ols(y3,x3);
prt(ols3); figure(4); plt(ols3); xlabel('1996:II-2011:IV');
print('./App_3_ols3.pdf','-dpdf')

% 2.d
yt = [ols1.yhat;ols2.yhat;ols3.yhat];
figure;
plot(t,yUS,'k',t,yt,'r');
legend('PIB','tendance');
print('./App_3_yUSyt.pdf','-dpdf')

% 2.e
yUSc = yUS - yt ;
plot(t,yUSc);
print('./App_3_yUSc.pdf','-dpdf')

% 2.f
yUS1 = yUS - X*b ;
plot(t,yUSc,'r',t,yUS1,'k');
legend('tendance coudee','tendance lineaire');
print('./App_3_yUS1.pdf','-dpdf')

% 3.a
e = randn(1000,1);

% 3.b
sacf(e,30);
print('./App_3_sacfe.pdf','-dpdf')

% 3.c
spacf(e,30);
print('./App_3_spacfe.pdf','-dpdf')

% 3.d
x = zeros(1000,1);
x(1) = e(1);
for ii=2:1000, x(ii) = .8*x(ii-1) + e(ii) ;	end;

% 3.e
sacf(x,30);
print('./App_3_sacfx.pdf','-dpdf')

spacf(x,30);
print('./App_3_spacfx.pdf','-dpdf')

% 3.f
m = zeros(1000,1);
m(1) = e(1);
for ii=2:1000, m(ii) = e(ii) - .9*e(ii-1) ;	end;
sacf(m,30);
print('./App_3_sacfm.pdf','-dpdf')

spacf(m,30);
print('./App_3_spacfm.pdf','-dpdf')

% 3.g
sacf(yUSc,30);
print('./App_3_sacf_yUSc.pdf','-dpdf')
spacf(yUSc,30);
print('./App_3_spacf_yUSc.pdf','-dpdf')

%%%%%%%%%%%%
% 3.f
armaUS = armaxfilter(yUSc,0,2,2,[],[]);
bigshow2(armaUS(1:2),armaUS(3:4));
print('./App_3_armaUS.pdf','-dpdf')

% 4.a
yUSthp = hptrend(yUS,1600);
figure;plot(t,yt,'r',t,yUSthp,'k');
legend('tendance coudee','filtre HP');
print('./App_3_yUSthp.pdf','-dpdf')

% 4.b
yUShp = yUS - yUSthp ;
armaHP = armaxfilter(yUShp,0,2,2,[],[]);
bigshow2(armaHP(1:2),armaHP(3:4));
print('./App_3_armaHP.pdf','-dpdf')

% 4.c
yUSbp = bpass(yUS,12,32);
armaBP = armaxfilter(yUSbp,0,2,2,[],[]);
bigshow2(armaBP(1:2),armaBP(3:4));
print('./App_3_armaBP.pdf','-dpdf')


% effacer les fichiers auxiliaires inutiles
delete g1.mat g2.mat g3.mat H.dat